import stldim

def test_empty_varname():
    assert stldim.get_varname("test.stl", "") == "test_stl"

def test_none_varname():
    assert stldim.get_varname("test.stl", None) == "test_stl"

def test_varname():
    assert stldim.get_varname("test.stl", "foobar") == "foobar"


import stldim
def test_plain():
    print(dir(stldim))
    assert stldim.sanitize_filename("test.stl") == "test_stl"

def test_spaces():
    assert stldim.sanitize_filename("test test.stl") == "test_test_stl"

def test_special_chars():
    assert stldim.sanitize_filename("test!@#$%^&*().stl") == "test___________stl"

def test_leading_numbers():
    assert stldim.sanitize_filename("11test.stl") == "__test_stl"

def test_trailing_numbers():
    assert stldim.sanitize_filename("test11.stl") == "test11_stl"

def test_no_extension():
    assert stldim.sanitize_filename("test") == "test"

def test_subdirectory():
    assert stldim.sanitize_filename("subdir/test.stl") == "test_stl"


import stldim
from stl import mesh
import pytest

def test_3dbenchy():
    stl_dimensions = stldim.get_stl_dimensions("tests/3DBenchy.stl")

    assert pytest.approx(stl_dimensions['minx']) == -29.176
    assert pytest.approx(stl_dimensions['maxx']) == 30.825
    assert pytest.approx(stl_dimensions['miny']) == -15.502
    assert pytest.approx(stl_dimensions['maxy']) == 15.502
    assert pytest.approx(stl_dimensions['minz']) == 0.0
    assert pytest.approx(stl_dimensions['maxz']) == 48.0



import stldim
from stl import mesh
import pytest

def test_3dbenchy():
    main_body = mesh.Mesh.from_file("tests/3DBenchy.stl")
    
    minx, maxx, miny, maxy, minz, maxz = stldim.find_mins_maxs(main_body)
    
    assert pytest.approx(minx) == -29.176
    assert pytest.approx(maxx) == 30.825
    assert pytest.approx(miny) == -15.502
    assert pytest.approx(maxy) == 15.502
    assert pytest.approx(minz) == 0.0
    assert pytest.approx(maxz) == 48.0



#!/usr/bin/env python3
"""
    stldim - Get dimensions of an STL file
    Usage:
        stldim.py [options] <stlfile>

    Options:
        -h --help       Show this screen.
        --version       Show version.
        --name=<name>   Name of the object [defaults to the filename with non alpha-numeric characters replaced with underscores].
"""

import argparse
import os
import sys

import stldim.version as version
from stldim import get_varname, get_stl_dimensions

def main():
    parser = argparse.ArgumentParser(prog="stldim",
                                     description="Get dimensions of an STL file")

    parser.add_argument("stlfile", type=str, help="Path to the STL file")
    parser.add_argument("--version", action="version",
                        help="Show version", version=version.__str__)
    parser.add_argument("--name", type=str, default=None,
                        help="Name of the object (defaults to filename with special characters replaced by underscores")

    args = parser.parse_args()

    if not os.path.exists(args.stlfile):
        sys.exit(f'ERROR: file args.stlfile was not found!')
    varname = get_varname(args.stlfile, args.name)

    stl_dimensions = get_stl_dimensions(args.stlfile)


# the logic is easy from there

    print("// File:", args.stlfile)
    lst = ['obj =("', args.stlfile, '");']
    obj = ['\t\timport("', args.stlfile, '");']

    print("// X size:", stl_dimensions['xsize'])
    print(f"{varname}_xsize = {stl_dimensions['xsize']};")
    print("// Y size:", stl_dimensions['ysize'])
    print(f"{varname}_ysize = {stl_dimensions['ysize']};")
    print("// Z size:", stl_dimensions['zsize'])
    print(f"{varname}_zsize = {stl_dimensions['zsize']};")
    print("// X position:", stl_dimensions['minx'])
    print(f"{varname}_xposition = {stl_dimensions['minx']};")
    print("// Y position:", stl_dimensions['miny'])
    print(f"{varname}_yposition = {stl_dimensions['miny']};")
    print("// Z position:", stl_dimensions['minz'])
    print(f"{varname}_zposition = {stl_dimensions['minz']};")

    # --------------------
    print("NE=1; NW=2; SW=3; SE=4; CTR=5; CTRXY=6;")

    print(f"module {varname}_obj2origin (where) {{")
    print("\tif (where == NE) {")
    print(f"\t\t{varname}_objNE ();")
    print("\t}")
    print("")

    print("\tif (where == NW) {")
    print("\t\ttranslate([", -stl_dimensions['xsize'], ",", 0, ",", 0, "])")
    print(f"\t\t{varname}_objNE ();")
    print("\t}")
    print("")

    print("\tif (where == SW) {")
    print("\t\ttranslate([", -stl_dimensions['xsize'],
          ",", -stl_dimensions['ysize'], ",", 0, "])")
    print(f"\t\t{varname}_objNE ();")
    print("\t}")
    print("")

    print("\tif (where == SE) {")
    print(
        "\t\ttranslate([", 0, ",", -stl_dimensions['ysize'], ",", 0, ",", "])")
    print(f"\t\t{varname}_objNE ();")
    print("\t}")
    print("")

    print("\tif (where == CTR) {")
    print("\ttranslate([", -stl_dimensions['midx'], ",", -
          stl_dimensions['midy'], ",", -stl_dimensions['midz'], "])")
    print(f"\t\t{varname}_objNE ();")
    print("\t}")
    print("")

    print("\tif (where == CTRXY) {")
    print("\ttranslate([", -stl_dimensions['midx'],
          ",", -stl_dimensions['midy'], ",", 0, "])")
    print(f"\t\t{varname}_objNE ();")
    print("\t}")
    print("}")
    print("")

    print(f"module {varname}_objNE () {{")
    print("\ttranslate([", -stl_dimensions['minx'], ",", -
          stl_dimensions['miny'], ",", -stl_dimensions['minz'], "])")
    print("".join(obj))
    print("}")


if __name__ == '__main__':
    main()



import os
import re

import stl
from stl import mesh


def sanitize_filename(stlfile):
    """
    Replace every non-alphanumeric character with an underscore
    """
    
    sanitized = re.sub(r'\W', '_', os.path.basename(stlfile)).lower()
    match = re.search(r"\D", sanitized)
    if match:
        return "_" * (match.start()) + sanitized[match.start():]
    return sanitized


def get_varname(filename, name):
    if name:
        return name
    else:
        return sanitize_filename(filename)


def find_mins_maxs(obj):
    minx = min([p[stl.Dimension.X] for p in obj.points])
    miny = min([p[stl.Dimension.Y] for p in obj.points])
    minz = min([p[stl.Dimension.Z] for p in obj.points])
    maxx = max([p[stl.Dimension.X] for p in obj.points])
    maxy = max([p[stl.Dimension.Y] for p in obj.points])
    maxz = max([p[stl.Dimension.Z] for p in obj.points])

    return minx, maxx, miny, maxy, minz, maxz


def get_stl_dimensions(stlfile):
    # find the max dimensions, so we can know the bounding box, getting the height,
    # width, length (because these are the step size)...
    stl_dimensions = {}

    main_body = mesh.Mesh.from_file(stlfile)

    stl_dimensions['minx'], stl_dimensions['maxx'], stl_dimensions['miny'], stl_dimensions[
        'maxy'], stl_dimensions['minz'], stl_dimensions['maxz'] = find_mins_maxs(main_body)

    stl_dimensions['minx'] = round(stl_dimensions['minx'], 3)
    stl_dimensions['maxx'] = round(stl_dimensions['maxx'], 3)
    stl_dimensions['miny'] = round(stl_dimensions['miny'], 3)
    stl_dimensions['maxy'] = round(stl_dimensions['maxy'], 3)
    stl_dimensions['minz'] = round(stl_dimensions['minz'], 3)
    stl_dimensions['maxz'] = round(stl_dimensions['maxz'], 3)

    stl_dimensions['xsize'] = round(
        stl_dimensions['maxx']-stl_dimensions['minx'], 3)
    stl_dimensions['ysize'] = round(
        stl_dimensions['maxy']-stl_dimensions['miny'], 3)
    stl_dimensions['zsize'] = round(
        stl_dimensions['maxz']-stl_dimensions['minz'], 3)

    stl_dimensions['midx'] = round(stl_dimensions['xsize']/2, 3)
    stl_dimensions['midy'] = round(stl_dimensions['ysize']/2, 3)
    stl_dimensions['midz'] = round(stl_dimensions['zsize']/2, 3)

    return stl_dimensions
